@@ -212,7 +212,8 @@ TALER_MHD_redirect_with_oauth_status (
       GNUNET_break (0);
       return MHD_NO;
     }
-    TALER_MHD_add_global_headers (response);
+    TALER_MHD_add_global_headers (response,
+                                  false);
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (response,
                                            MHD_HTTP_HEADER_CONTENT_TYPE,
@@ -265,7 +266,8 @@ TALER_MHD_redirect_with_oauth_status (
                                  oauth_error_uri)));
 
     response = TALER_MHD_make_json_steal (args);
-    TALER_MHD_add_global_headers (response);
+    TALER_MHD_add_global_headers (response,
+                                  false);
     http_status = MHD_HTTP_TOO_MANY_REQUESTS;
   }
 
