@@ -89,7 +89,7 @@ CH_code_to_nonce (const char *code,
  * @return a MHD result code
  */
 MHD_RESULT
-TALER_MHD_reply_with_oauth_error (
+CH_reply_with_oauth_error (
   struct MHD_Connection *connection,
   unsigned int http_status,
   const char *oauth_error,
