@@ -1,4 +1,4 @@
 #!/bin/sh
 # This file is in the public domain.
-EMAIL=$(echo "$1" | jq -r .email)
+EMAIL=$(echo "$1" | jq -r .CONTACT_EMAIL)
 exec mail -s "KYC Challenger" -r noreply "$EMAIL"
