@@ -14,7 +14,9 @@
   Challenger; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
- *
+ * @file include/challenger_database_lib.h
+ * @brief database helper functions for postgres used by challenger
+ * @author Christian Grothoff
  */
 #ifndef CHALLENGER_DB_LIB_H
 #define CHALLENGER_DB_LIB_H
@@ -26,10 +28,14 @@
  * Initialize the plugin.
  *
  * @param cfg configuration to use
+ * @param skip_preflight true if we should skip the usual
+ *   preflight check which assures us that the DB is actually
+ *   operational; only challenger-dbinit should use true here.
  * @return NULL on failure
  */
 struct CHALLENGER_DatabasePlugin *
-CHALLENGER_DB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg);
+CHALLENGER_DB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                           bool skip_preflight);
 
 
 /**
