@@ -24,12 +24,12 @@
 #include <gnunet/gnunet_util_lib.h>
 
 /**
- * Return default project data used by Challenger.
+ * Return project data used by Challenger.
  *
  * @return project data for challenger
  */
 const struct GNUNET_OS_ProjectData *
-CHALLENGER_project_data_default (void);
+CHALLENGER_project_data (void);
 
 
 #endif
